@@ -25,6 +25,10 @@
 
 #include "../SDL_tray_utils.h"
 
+void SDL_UpdateTrays(void)
+{
+}
+
 SDL_Tray *SDL_CreateTray(SDL_Surface *icon, const char *tooltip)
 {
     SDL_Unsupported();
@@ -33,29 +37,27 @@ SDL_Tray *SDL_CreateTray(SDL_Surface *icon, const char *tooltip)
 
 void SDL_SetTrayIcon(SDL_Tray *tray, SDL_Surface *icon)
 {
-    SDL_Unsupported();
 }
 
 void SDL_SetTrayTooltip(SDL_Tray *tray, const char *tooltip)
 {
-    SDL_Unsupported();
 }
 
 SDL_TrayMenu *SDL_CreateTrayMenu(SDL_Tray *tray)
 {
-    SDL_Unsupported();
+    SDL_InvalidParamError("tray");
     return NULL;
 }
 
 SDL_TrayMenu *SDL_GetTrayMenu(SDL_Tray *tray)
 {
-    SDL_Unsupported();
+    SDL_InvalidParamError("tray");
     return NULL;
 }
 
 SDL_TrayMenu *SDL_CreateTraySubmenu(SDL_TrayEntry *entry)
 {
-    SDL_Unsupported();
+    SDL_InvalidParamError("entry");
     return NULL;
 }
 
@@ -66,57 +68,50 @@ SDL_TrayMenu *SDL_GetTraySubmenu(SDL_TrayEntry *entry)
 
 const SDL_TrayEntry **SDL_GetTrayEntries(SDL_TrayMenu *menu, int *size)
 {
-    SDL_Unsupported();
+    SDL_InvalidParamError("menu");
     return NULL;
 }
 
 void SDL_RemoveTrayEntry(SDL_TrayEntry *entry)
 {
-    SDL_Unsupported();
 }
 
 SDL_TrayEntry *SDL_InsertTrayEntryAt(SDL_TrayMenu *menu, int pos, const char *label, SDL_TrayEntryFlags flags)
 {
-    SDL_Unsupported();
+    SDL_InvalidParamError("menu");
     return NULL;
 }
 
 void SDL_SetTrayEntryLabel(SDL_TrayEntry *entry, const char *label)
 {
-    SDL_Unsupported();
 }
 
 const char *SDL_GetTrayEntryLabel(SDL_TrayEntry *entry)
 {
-    SDL_Unsupported();
+    SDL_InvalidParamError("entry");
     return NULL;
 }
 
 void SDL_SetTrayEntryChecked(SDL_TrayEntry *entry, bool checked)
 {
-    SDL_Unsupported();
 }
 
 bool SDL_GetTrayEntryChecked(SDL_TrayEntry *entry)
 {
-    SDL_Unsupported();
-    return false;
+    return SDL_InvalidParamError("entry");
 }
 
 void SDL_SetTrayEntryEnabled(SDL_TrayEntry *entry, bool enabled)
 {
-    SDL_Unsupported();
 }
 
 bool SDL_GetTrayEntryEnabled(SDL_TrayEntry *entry)
 {
-    SDL_Unsupported();
-    return false;
+    return SDL_InvalidParamError("entry");
 }
 
 void SDL_SetTrayEntryCallback(SDL_TrayEntry *entry, SDL_TrayCallback callback, void *userdata)
 {
-    SDL_Unsupported();
 }
 
 void SDL_ClickTrayEntry(SDL_TrayEntry *entry)
@@ -125,25 +120,24 @@ void SDL_ClickTrayEntry(SDL_TrayEntry *entry)
 
 SDL_TrayMenu *SDL_GetTrayEntryParent(SDL_TrayEntry *entry)
 {
-    SDL_Unsupported();
+    SDL_InvalidParamError("entry");
     return NULL;
 }
 
 SDL_TrayEntry *SDL_GetTrayMenuParentEntry(SDL_TrayMenu *menu)
 {
-    SDL_Unsupported();
+    SDL_InvalidParamError("menu");
     return NULL;
 }
 
 SDL_Tray *SDL_GetTrayMenuParentTray(SDL_TrayMenu *menu)
 {
-    SDL_Unsupported();
+    SDL_InvalidParamError("menu");
     return NULL;
 }
 
 void SDL_DestroyTray(SDL_Tray *tray)
 {
-    SDL_Unsupported();
 }
 
 #endif // !SDL_PLATFORM_MACOS
